@@ -1,4 +1,6 @@
 import os
+import json
+import numpy as np
 
 from .. import util
 from .. import resolvers
@@ -21,7 +23,6 @@ from .misc import _misc_loaders
 from .gltf import _gltf_loaders
 from .xaml import _xaml_loaders
 from .binvox import _binvox_loaders
-from .assimp import _assimp_loaders
 from .threemf import _three_loaders
 from .openctm import _ctm_loaders
 from .threedxml import _threedxml_loaders
@@ -34,7 +35,9 @@ except BaseException as E:
     from ..exceptions import closure
     load_path = closure(E)
     # no path formats available
-    def path_formats(): return []
+
+    def path_formats():
+        return set()
 
 
 def mesh_formats():
@@ -47,7 +50,7 @@ def mesh_formats():
       Extensions of available mesh loaders,
       i.e. 'stl', 'ply', etc.
     """
-    return list(mesh_loaders.keys())
+    return set(mesh_loaders.keys())
 
 
 def available_formats():
@@ -60,9 +63,10 @@ def available_formats():
         Extensions of available loaders
         i.e. 'stl', 'ply', 'dxf', etc.
     """
+    # set
     loaders = mesh_formats()
-    loaders.extend(path_formats())
-    loaders.extend(compressed_loaders.keys())
+    loaders.update(path_formats())
+    loaders.update(compressed_loaders.keys())
     return loaders
 
 
@@ -295,29 +299,44 @@ def load_compressed(file_obj,
             else:
                 available = mesh_formats()
 
+        meta_archive = {}
         for name, data in files.items():
-            # only load formats that we support
-            compressed_type = util.split_extension(name).lower()
-            if compressed_type not in available:
-                # don't raise an exception, just try the next one
-                continue
-            # store the file name relative to the archive
-            metadata['file_name'] = (archive_name + '/' +
-                                     os.path.basename(name))
-            # load the individual geometry
-            loaded = load(file_obj=data,
-                          file_type=compressed_type,
-                          resolver=resolver,
-                          metadata=metadata,
-                          **kwargs)
+            try:
+                # only load formats that we support
+                compressed_type = util.split_extension(
+                    name).lower()
 
-            # some loaders return multiple geometries
-            if util.is_sequence(loaded):
-                # if the loader has returned a list of meshes
-                geometries.extend(loaded)
-            else:
-                # if the loader has returned a single geometry
-                geometries.append(loaded)
+                # if file has metadata type include it
+                if compressed_type in 'yaml':
+                    import yaml
+                    meta_archive[name] = yaml.safe_load(data)
+                elif compressed_type in 'json':
+                    import json
+                    meta_archive[name] = json.loads(data)
+
+                if compressed_type not in available:
+                    # don't raise an exception, just try the next one
+                    continue
+                # store the file name relative to the archive
+                metadata['file_name'] = (archive_name + '/' +
+                                         os.path.basename(name))
+                # load the individual geometry
+                loaded = load(file_obj=data,
+                              file_type=compressed_type,
+                              resolver=resolver,
+                              metadata=metadata,
+                              **kwargs)
+
+                # some loaders return multiple geometries
+                if util.is_sequence(loaded):
+                    # if the loader has returned a list of meshes
+                    geometries.extend(loaded)
+                else:
+                    # if the loader has returned a single geometry
+                    geometries.append(loaded)
+            except BaseException:
+                log.debug('failed to load file in zip',
+                          exc_info=True)
 
     finally:
         # if we opened the file in this function
@@ -328,6 +347,10 @@ def load_compressed(file_obj,
     # append meshes or scenes into a single Scene object
     result = append_scenes(geometries)
 
+    # append any archive metadata files
+    if isinstance(result, Scene):
+        result.metadata.update(meta_archive)
+
     return result
 
 
@@ -411,8 +434,25 @@ def load_kwargs(*args, **kwargs):
 
         if 'base_frame' in kwargs:
             scene.graph.base_frame = kwargs['base_frame']
-        if 'metadata' in kwargs:
+        metadata = kwargs.get('metadata')
+        if isinstance(metadata, dict):
             scene.metadata.update(kwargs['metadata'])
+        elif isinstance(metadata, str):
+            # some ways someone might have encoded a string
+            # note that these aren't evaluated until we
+            # actually call the lambda in the loop
+            candidates = [
+                lambda: json.loads(metadata),
+                lambda: json.loads(metadata.replace("'", '"'))]
+            for c in candidates:
+                try:
+                    scene.metadata.update(c())
+                    break
+                except BaseException:
+                    pass
+        elif metadata is not None:
+            log.warning('unloadable metadata')
+
         return scene
 
     def handle_mesh():
@@ -439,8 +479,9 @@ def load_kwargs(*args, **kwargs):
 
     def handle_path():
         from ..path import Path2D, Path3D
-        shape = kwargs['vertices'].shape
-
+        shape = np.shape(kwargs['vertices'])
+        if len(shape) < 2:
+            return Path2D()
         if shape[1] == 2:
             return Path2D(**kwargs)
         elif shape[1] == 3:
@@ -619,9 +660,6 @@ compressed_loaders = {'zip': load_compressed,
 
 # map file_type to loader function
 mesh_loaders = {}
-# assimp has a lot of loaders, but they are all quite slow
-# load first and replace with native loaders where possible
-mesh_loaders.update(_assimp_loaders)
 mesh_loaders.update(_misc_loaders)
 mesh_loaders.update(_stl_loaders)
 mesh_loaders.update(_ctm_loaders)
