@@ -29,8 +29,9 @@ def plane_transform(origin, normal):
         Transformation matrix to move points onto XY plane
     """
     transform = align_vectors(normal, [0, 0, 1])
-    transform[0:3, 3] = -np.dot(transform,
-                                np.append(origin, 1))[0:3]
+    if origin is not None:
+        transform[:3, 3] = -np.dot(
+            transform, np.append(origin, 1))[:3]
     return transform
 
 
@@ -146,26 +147,50 @@ def vector_angle(pairs):
     return angles
 
 
-def triangulate_quads(quads):
+def triangulate_quads(quads, dtype=np.int64):
     """
-    Given a set of quad faces, return them as triangle faces.
+    Given an array of quad faces return them as triangle faces,
+    also handles pure triangles and mixed triangles and quads.
 
     Parameters
     -----------
     quads: (n, 4) int
-      Vertex indices of quad faces
+      Vertex indices of quad faces.
 
     Returns
     -----------
     faces : (m, 3) int
-      Vertex indices of triangular faces
+      Vertex indices of triangular faces.c
     """
-    if len(quads) == 0:
-        return quads
     quads = np.asanyarray(quads)
-    faces = np.vstack((quads[:, [0, 1, 2]],
-                       quads[:, [2, 3, 0]]))
-    return faces
+    if len(quads) == 0:
+        return quads.astype(dtype)
+    elif len(quads.shape) == 2 and quads.shape[1] == 3:
+        # if they are just triangles return immediately
+        return quads.astype(dtype)
+    elif len(quads.shape) == 2 and quads.shape[1] == 4:
+        # if they are just quads stack and return
+        return np.vstack((quads[:, [0, 1, 2]],
+                          quads[:, [2, 3, 0]])).astype(dtype)
+    else:
+        # mixed tris, and quads, and other so filter and handle
+        tri = np.array([i for i in quads if len(i) == 3])
+        quad = np.array([i for i in quads if len(i) == 4])
+        # triangulate arbitrary polygons as fans
+        poly = [[[f[0], f[i + 1], f[i + 2]]
+                 for i in range(len(f) - 2)]
+                for f in quads if len(f) > 4]
+
+        if len(quad) == 0 and len(poly) == 0:
+            return tri.astype(dtype)
+        if len(poly) > 0:
+            poly = np.vstack(poly)
+        if len(quad) > 0:
+            quad = np.vstack((quad[:, [0, 1, 2]],
+                              quad[:, [2, 3, 0]]))
+        # combine triangulated quads with triangles
+        return util.vstack_empty([
+            tri, quad, poly]).astype(dtype)
 
 
 def vertex_face_indices(vertex_count,
@@ -428,7 +453,7 @@ def index_sparse(columns, indices, data=None):
 
     shape = (columns, len(indices))
     if data is None:
-        data = np.ones(len(col), dtype=np.bool)
+        data = np.ones(len(col), dtype=bool)
 
     # assemble into sparse matrix
     matrix = scipy.sparse.coo_matrix((data, (row, col)),
