@@ -5,35 +5,92 @@ entities.py
 Basic geometric primitives which only store references to
 vertex indices rather than vertices themselves.
 """
-
 import numpy as np
 
-import copy
-
+from copy import deepcopy
 from .arc import discretize_arc, arc_center
 from .curve import discretize_bezier, discretize_bspline
 
 from .. import util
+from ..util import ABC
 
 
-class Entity(object):
+class Entity(ABC):
 
     def __init__(self,
                  points,
                  closed=None,
                  layer=None,
+                 metadata=None,
+                 color=None,
                  **kwargs):
-
+        # points always reference vertex indices and are int
         self.points = np.asanyarray(points, dtype=np.int64)
+        # save explicit closed
         if closed is not None:
             self.closed = closed
-        self.layer = layer
+        # save the passed layer
+        if layer is not None:
+            self.layer = layer
+        if metadata is not None:
+            self.metadata.update(metadata)
+
+        self._cache = {}
+
+        # save the passed color
+        self.color = color
+        # save any other kwargs for general use
         self.kwargs = kwargs
 
+    @property
+    def metadata(self):
+        """
+        Get any metadata about the entity.
+
+        Returns
+        ---------
+        metadata : dict
+          Bag of properties.
+        """
+        if not hasattr(self, '_metadata'):
+            self._metadata = {}
+        # note that we don't let a new dict be assigned
+        return self._metadata
+
+    @property
+    def layer(self):
+        """
+        Set the layer the entity resides on as a shortcut
+        to putting it in the entity metadata.
+
+        Returns
+        ----------
+        layer : any
+          Hashable layer identifier.
+        """
+        return self.metadata.get('layer')
+
+    @layer.setter
+    def layer(self, value):
+        """
+        Set the current layer of the entity.
+
+        Returns
+        ----------
+        layer : any
+          Hashable layer indicator
+        """
+        self.metadata['layer'] = value
+
     def to_dict(self):
         """
         Returns a dictionary with all of the information
         about the entity.
+
+        Returns
+        -----------
+        as_dict : dict
+          Has keys 'type', 'points', 'closed'
         """
         return {'type': self.__class__.__name__,
                 'points': self.points.tolist(),
@@ -44,6 +101,11 @@ class Entity(object):
         """
         If the first point is the same as the end point
         the entity is closed
+
+        Returns
+        -----------
+        closed : bool
+          Is the entity closed or not?
         """
         closed = (len(self.points) > 2 and
                   self.points[0] == self.points[-1])
@@ -84,8 +146,10 @@ class Entity(object):
         in self.points aren't the endpoints of the curve you need to
         implement this function for your class.
 
-        self.points = [0,1,2]
-        returns:      [0,2]
+        Returns
+        -------------
+        ends : (2,) int
+          Indices of the two end points of the entity
         """
         return self.points[[0, -1]]
 
@@ -104,6 +168,12 @@ class Entity(object):
     def reverse(self, direction=-1):
         """
         Reverse the current entity in place.
+
+        Parameters
+        ----------------
+        direction : int
+          If positive will not touch direction
+          If negative will reverse self.points
         """
         if direction < 0:
             self._direction = -1
@@ -112,7 +182,17 @@ class Entity(object):
 
     def _orient(self, curve):
         """
-        Reverse a curve if a flag is set
+        Reverse a curve if a flag is set.
+
+        Parameters
+        --------------
+        curve : (n, dimension) float
+          Curve made up of line segments in space
+
+        Returns
+        ------------
+        orient : (n, dimension) float
+          Original curve, but possibly reversed
         """
         if hasattr(self, '_direction') and self._direction < 0:
             return curve[::-1]
@@ -124,11 +204,13 @@ class Entity(object):
 
         Parameters
         -----------
-        vertices: (n,dimension) float, vertices in space
+        vertices : (n, dimension) float
+          Vertices in space
 
         Returns
         -----------
-        bounds: (2, dimension) float, (min, max) coordinate of AABB
+        bounds : (2, dimension) float
+          Coordinates of AABB, in (min, max) form
         """
         bounds = np.array([vertices[self.points].min(axis=0),
                            vertices[self.points].max(axis=0)])
@@ -138,25 +220,48 @@ class Entity(object):
         """
         Return the total length of the entity.
 
+        Parameters
+        --------------
+        vertices : (n, dimension) float
+          Vertices in space
+
         Returns
         ---------
-        length: float, total length of entity
+        length : float
+          Total length of entity
         """
-        length = ((np.diff(self.discrete(vertices),
-                           axis=0)**2).sum(axis=1)**.5).sum()
+        diff = np.diff(self.discrete(vertices), axis=0) ** 2
+        length = (np.dot(diff, [1] * vertices.shape[1]) ** 0.5).sum()
         return length
 
     def explode(self):
         """
         Split the entity into multiple entities.
+
+        Returns
+        ------------
+        explode : list of Entity
+          Current entity split into multiple entities.
         """
-        return [self]
+        return [self.copy()]
 
     def copy(self):
         """
         Return a copy of the current entity.
+
+        Returns
+        ------------
+        copied : Entity
+          Copy of current entity
         """
-        return copy.deepcopy(self)
+        copied = deepcopy(self)
+        # only copy metadata if set
+        if hasattr(self, '_metadata'):
+            copied._metadata = deepcopy(self._metadata)
+            # check for very annoying subtle copy failures
+            assert id(copied._metadata) != id(self._metadata)
+        assert id(copied.points) != id(self.points)
+        return copied
 
     def __hash__(self):
         """
@@ -167,10 +272,17 @@ class Entity(object):
         hashed : int
             Hash of current class name, points, and closed
         """
-        hashed = hash(self._bytes())
-        return hashed
+        return hash(self._bytes())
 
     def _bytes(self):
+        """
+        Get hashable bytes that define the current entity.
+
+        Returns
+        ------------
+        data : bytes
+          Hashable data defining the current entity
+        """
         # give consistent ordering of points for hash
         if self.points[0] > self.points[-1]:
             return (self.__class__.__name__.encode('utf-8') +
@@ -191,7 +303,10 @@ class Text(Entity):
                  height=None,
                  vector=None,
                  normal=None,
-                 align=None):
+                 align=None,
+                 layer=None,
+                 color=None,
+                 metadata=None):
         """
         An entity for text labels.
 
@@ -215,12 +330,21 @@ class Text(Entity):
         """
         # where is text placed
         self.origin = origin
-
+        # what direction is the text pointing
         self.vector = vector
+        # what is the normal of the text plane
         self.normal = normal
-
         # how high is the text entity
         self.height = height
+        # what layer is the entity on
+        if layer is not None:
+            self.layer = layer
+
+        if metadata is not None:
+            self.metadata.update(metadata)
+
+        # what color is the entity
+        self.color = color
 
         # None or (2,) str
         if align is None:
@@ -323,6 +447,7 @@ class Text(Entity):
         # get rotation angle in degrees
         angle = np.degrees(self.angle(vertices))
 
+        # TODO: handle text size better
         plt.text(*vertices[self.origin],
                  s=self.text,
                  rotation=angle,
@@ -408,8 +533,7 @@ class Line(Entity):
         discrete: (m, dimension) float
           Path in space composed of line segments
         """
-        discrete = self._orient(vertices[self.points])
-        return discrete
+        return self._orient(vertices[self.points])
 
     @property
     def is_valid(self):
@@ -433,10 +557,12 @@ class Line(Entity):
         ----------
         exploded: (n,) Line entities
         """
+        # copy over the current layer
+        layer = self.layer
         points = np.column_stack((
             self.points,
             self.points)).ravel()[1:-1].reshape((-1, 2))
-        exploded = [Line(i) for i in points]
+        exploded = [Line(i, layer=layer) for i in points]
         return exploded
 
     def _bytes(self):
@@ -459,9 +585,7 @@ class Arc(Entity):
         closed : bool
           If set True, Arc will be a closed circle
         """
-        if hasattr(self, '_closed'):
-            return self._closed
-        return False
+        return getattr(self, '_closed', False)
 
     @closed.setter
     def closed(self, value):
@@ -490,10 +614,34 @@ class Arc(Entity):
 
     def _bytes(self):
         # give consistent ordering of points for hash
-        if self.points[0] > self.points[-1]:
-            return b'Arc' + bytes(self.closed) + self.points.tobytes()
-        else:
-            return b'Arc' + bytes(self.closed) + self.points[::-1].tobytes()
+        order = int(self.points[0] > self.points[-1]) * 2 - 1
+        return b'Arc' + bytes(self.closed) + self.points[::order].tobytes()
+
+    def length(self, vertices):
+        """
+        Return the arc length of the 3-point arc.
+
+        Parameter
+        ----------
+        vertices : (n, d) float
+          Vertices for overall drawing.
+
+        Returns
+        -----------
+        length : float
+          Length of arc.
+        """
+        # find the actual radius and angle span
+        if self.closed:
+            # we don't need the angular span as
+            # it's indicated as a closed circle
+            fit = self.center(
+                vertices, return_normal=False, return_angle=False)
+            return np.pi * fit['radius'] * 4
+        # get the angular span of the circular arc
+        fit = self.center(
+            vertices, return_normal=False, return_angle=True)
+        return fit['span'] * fit['radius'] * 2
 
     def discrete(self, vertices, scale=1.0):
         """
@@ -508,29 +656,30 @@ class Arc(Entity):
 
         Returns
         -------------
-        discrete: (m, dimension) float, linear path in space
+        discrete : (m, dimension) float
+          Path in space made up of line segments
         """
-        discrete = discretize_arc(vertices[self.points],
-                                  close=self.closed,
-                                  scale=scale)
-        return self._orient(discrete)
 
-    def center(self, vertices):
+        return self._orient(discretize_arc(
+            vertices[self.points],
+            close=self.closed,
+            scale=scale))
+
+    def center(self, vertices, **kwargs):
         """
         Return the center information about the arc entity.
 
         Parameters
         -------------
         vertices : (n, dimension) float
-            Vertices in space
+          Vertices in space
 
         Returns
         -------------
-        info: dict, with keys: 'radius'
-                               'center'
+        info : dict
+          With keys: 'radius', 'center'
         """
-        info = arc_center(vertices[self.points])
-        return info
+        return arc_center(vertices[self.points], **kwargs)
 
     def bounds(self, vertices):
         """
@@ -538,17 +687,19 @@ class Arc(Entity):
 
         Parameters
         -----------
-        vertices: (n,dimension) float, vertices in space
+        vertices: (n, dimension) float
+          Vertices in space
 
         Returns
         -----------
-        bounds: (2, dimension) float, (min, max) coordinate of AABB
+        bounds : (2, dimension) float
+          Coordinates of AABB in (min, max) form
         """
         if util.is_shape(vertices, (-1, 2)) and self.closed:
             # if we have a closed arc (a circle), we can return the actual bounds
             # this only works in two dimensions, otherwise this would return the
             # AABB of an sphere
-            info = self.center(vertices)
+            info = self.center(vertices, return_normal=False, return_angle=False)
             bounds = np.array([info['center'] - info['radius'],
                                info['center'] + info['radius']],
                               dtype=np.float64)
@@ -597,10 +748,10 @@ class Bezier(Curve):
         discrete : (m, 2) or (m, 3) float
           Curve as line segments
         """
-        discrete = discretize_bezier(vertices[self.points],
-                                     count=count,
-                                     scale=scale)
-        return self._orient(discrete)
+        return self._orient(discretize_bezier(
+            vertices[self.points],
+            count=count,
+            scale=scale))
 
 
 class BSpline(Curve):
@@ -608,15 +759,22 @@ class BSpline(Curve):
     An open or closed B- Spline.
     """
 
-    def __init__(self, points,
+    def __init__(self,
+                 points,
                  knots,
-                 closed=None,
                  layer=None,
+                 metadata=None,
+                 color=None,
                  **kwargs):
         self.points = np.asanyarray(points, dtype=np.int64)
         self.knots = np.asanyarray(knots, dtype=np.float64)
-        self.layer = layer
+        if layer is not None:
+            self.layer = layer
+        if metadata is not None:
+            self.metadata.update(metadata)
+        self._cache = {}
         self.kwargs = kwargs
+        self.color = color
 
     def discrete(self, vertices, count=None, scale=1.0):
         """
