@@ -12,6 +12,13 @@ import numpy as np
 
 import pyglet
 
+# pyglet 2.0 is close to a re-write moving from fixed-function
+# to shaders and we will likely support it by forking an entirely
+# new viewer `trimesh.viewer.shaders` and then basically keeping
+# `windowed` around for backwards-compatibility with no changes
+if int(pyglet.version.split('.')[0]) >= 2:
+    raise ImportError('`trimesh.viewer.windowed` requires `pip install "pyglet<2"`')
+
 from .trackball import Trackball
 
 from .. import util
@@ -848,21 +855,10 @@ def geometry_hash(geometry):
     ------------
     hash : str
     """
-    if hasattr(geometry, 'crc'):
-        # for most of our trimesh objects
-        h = str(geometry.crc())
-    elif hasattr(geometry, 'md5'):
-        h = geometry.md5()
-    elif hasattr(geometry, 'tostring'):
-        # for unwrap ndarray objects
-        h = str(hash(geometry.tostring()))
-
+    h = str(hash(geometry))
     if hasattr(geometry, 'visual'):
         # if visual properties are defined
-        h += str(geometry.visual.crc())
-    elif hasattr(geometry, 'visual_crc'):
-        # paths do not use the visual attribute
-        h += str(geometry.colors_crc())
+        h += str(hash(geometry.visual))
 
     return h
 
